import { env, ReservedSQL, sql } from 'bun'
import { MIMEType } from 'util';

export enum MultiMediaType {
  video, image
}

export interface Attachment {
  urls: string[],
  type: MultiMediaType
};

export interface Post {
  post: string,
  post_time: string,
  hash: string,
  attachments: Attachment[],
}
interface SQLPostCast {
  hash: string, post: string, post_time: string, images: number[]
};
const SQLPostCast2Post = (obj: SQLPostCast) => {
  let x: Post =
  {
    post: obj.post,
    hash: obj.hash,
    post_time: obj.post_time,
    attachments: []
  };
  if (obj.images && obj.images.length > 0) {
    x.attachments.push(
      {
        type: MultiMediaType.image, urls: obj.images.map(img => {
          return `/img/${obj.hash}_${img}`
        })
      }
    )
  }
  return x
}

export interface Cursor {
  hash: string,
  post_time: string,
}


export class PostFetcher {
  conn: Promise<ReservedSQL>;
  constructor() {
    this.conn = sql.reserve();
  }
  async init() {
    await this.conn.then(async e => {
      await e`DECLARE post_ptr CURSOR WITH HOLD FOR SELECT niming.posts.*, niming.images.file_sequence 
        AS images FROM niming.posts 
        LEFT JOIN niming.images ON niming.images.hash=niming.posts.hash ORDER BY niming.posts.post_time DESC, niming.posts.hash ASC`
    }).catch(err => {
      console.log(err);
    })
  }

  async postgres_fetch() {
    let x: SQLPostCast[] = []
    if (this.conn) {
      x = await (await this.conn)`FETCH 10 IN post_ptr`;
    }
    return x.map(e => {
      return SQLPostCast2Post(e)
    })
  }
}

